/*

  This is a library / example program for the Colorduino v1.3, based
  on the Colorduino library: https://github.com/lincomatic/Colorduino

  It offers text display, text scrolling, simple color graphics (line
  graphics, rectangles, plots, circles), bitmap images, scrolling over
  background bitmap images, simple image effects such as rotation and
  scrolling, etc. See loop() demo at the end of this file.

  This software is (c) Alex Fester and Michael Wessel, January 2015: 
  http://www.michael-wessel.info 

  This software uses parts of the Adafruit-GFX library: 
    https://github.com/adafruit/Adafruit-GFX-Library/blob/master/Adafruit_GFX.cpp
  as well as parts of the ColorduinoScrollerLibrary: 
    https://github.com/maujabur/ColorduinoScrollerLibrary
  and the basic Colorduino library: 
    https://github.com/lincomatic/Colorduino

  The rationale for this software is as follows: neither the
  Adafruit-GFX-Library nor the ColorduinoScrollerLibrary were working
  for us. All graphics and scrolling operations were extremely slow,
  i.e.  scrolling one character took 20 seconds, and this wasn't
  caused by a delay in the scrolling function. Rather, we are
  suspecting that the (current) version of the Adafruit_GFX library on
  which the ColorduinoScrollerLibrary is based is not really
  compatible with the (current) version of the Colorduino. We hence
  significantly simplified and combined these two libaries into this
  piece of software. If you are having problems getting your
  Colorduino to work with the Adafruit_GFX library or the above text
  scrolling library, consider using this software. It should work
  right out of the box. We didn't bother making a library from it,
  though. Should be straight forward. Simply load this program into
  the Colorduino and adjust demo in loop() as required.

  This software is open source code. Copyright (c) Alex Fester &
  Michael Wessel.  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided that the
  following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 

*
* Note that this work is based on the Colorduino library, with the
* following license:
*

 Colorduino - Colorduino Library for Arduino
  Copyright (c) 2011-2012 Sam C. Lin <lincomatic@hotmail.com>
  based on C code by zzy@iteadstudio
    Copyright (c) 2010 zzy@IteadStudio.  All right reserved.
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*
* Note that this work is based on parts of the original
* Adafruit-GFX-Library, which is reproduced here as follows for
* license compliance:
*

  This is the core graphics library for all our displays, providing a common
  set of graphics primitives (points, lines, circles, etc.).  It needs to be
  paired with a hardware-specific library for each display device we carry
  (to handle the lower-level functions).
  Adafruit invests time and resources providing this open source code, please
  support Adafruit & open-source hardware by purchasing products from Adafruit!
  Copyright (c) 2013 Adafruit Industries.  All rights reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

*/

#include <Colorduino.h>
#include "glcdfont.c"

#define adagfxswap(a, b) { int8_t t = a; a = b; b = t; }

uint8_t red = 255;
uint8_t green = 255;
uint8_t blue = 255;

uint8_t bg_red = 0;
uint8_t bg_green = 0;
uint8_t bg_blue = 0;

uint8_t red_saved = 255;
uint8_t green_saved = 255;
uint8_t blue_saved = 255;

uint8_t bg_red_saved = 0;
uint8_t bg_green_saved = 0;
uint8_t bg_blue_saved = 0;

const uint8_t colors[][3] =
  { { 255, 255, 255}, // white 0
    { 255, 000, 000}, // red   1
    { 255, 140, 000}, // orange 2
    { 255, 255, 000}, // yellow 3
    { 000, 60, 000}, // green 4
    { 000, 000, 255}, // blue 5
    { 255, 000, 255}, // purple 6
    { 139, 67, 19}, // brown 7 
    { 000, 000, 000},  // black 8
    { 255, 255, 255} // snow white 9
  };
 
String heart[] = {
  "--------",
  "--11-11-",
  "-1111111",
  "-1111111",
  "--11111-",
  "---111--",
  "----1---",
  "--------"
};
    
String hi[] = {
  "55555555",
  "35535333",
  "35535535",
  "33335535",
  "35535535",
  "35535333",
  "55555555",
  "55555555"
};

void plot(int8_t x, int8_t y) {

  if (x < 0 || x > 7 || y < 0 || y > 7)
    return;

  Colorduino.SetPixel(x, y, red, green, blue);

}

void bg_plot(int8_t x, int8_t y) {

  if (x < 0 || x > 7 || y < 0 || y > 7)
    return;

  Colorduino.SetPixel(x, y, bg_red, bg_green, bg_blue);

}

void drawLine(int8_t x0, int8_t y0,
              int8_t x1, int8_t y1) {
  int8_t steep = abs(y1 - y0) > abs(x1 - x0);
  if (steep) {
    adagfxswap(x0, y0);
    adagfxswap(x1, y1);
  }

  if (x0 > x1) {
    adagfxswap(x0, x1);
    adagfxswap(y0, y1);
  }

  int8_t dx, dy;
  dx = x1 - x0;
  dy = abs(y1 - y0);

  int8_t err = dx / 2;
  int8_t ystep;

  if (y0 < y1) {
    ystep = 1;
  } else {
    ystep = -1;
  }

  for (; x0 <= x1; x0++) {
    if (steep) {
      plot(y0, x0);
    } else {
      plot(x0, y0);
    }
    err -= dy;
    if (err < 0) {
      y0 += ystep;
      err += dx;
    }
  }
}

void drawCircle(int8_t x0, int8_t y0, int8_t r) {

  int8_t f = 1 - r;
  int8_t ddF_x = 1;
  int8_t ddF_y = -2 * r;
  int8_t x = 0;
  int8_t y = r;

  plot(x0  , y0 + r);
  plot(x0  , y0 - r);
  plot(x0 + r, y0);
  plot(x0 - r, y0);

  while (x < y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f += ddF_y;
    }
    x++;
    ddF_x += 2;
    f += ddF_x;

    plot(x0 + x, y0 + y);
    plot(x0 - x, y0 + y);
    plot(x0 + x, y0 - y);
    plot(x0 - x, y0 - y);
    plot(x0 + y, y0 + x);
    plot(0 - y, y0 + x);
    plot(x0 + y, y0 - x);
    plot(x0 - y, y0 - x);
  }
}

void drawCircleHelper( int8_t x0, int8_t y0,
                       int8_t r, uint8_t cornername ) {
  int8_t f     = 1 - r;
  int8_t ddF_x = 1;
  int8_t ddF_y = -2 * r;
  int8_t x     = 0;
  int8_t y     = r;

  while (x < y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f     += ddF_y;
    }
    x++;
    ddF_x += 2;
    f     += ddF_x;
    if (cornername & 0x4) {
      plot(x0 + x, y0 + y);
      plot(x0 + y, y0 + x);
    }
    if (cornername & 0x2) {
      plot(x0 + x, y0 - y);
      plot(x0 + y, y0 - x);
    }
    if (cornername & 0x8) {
      plot(x0 - y, y0 + x);
      plot(x0 - x, y0 + y);
    }
    if (cornername & 0x1) {
      plot(x0 - y, y0 - x);
      plot(x0 - x, y0 - y);
    }
  }
}

void fillCircle(int8_t x0, int8_t y0, int8_t r) {
  drawFastVLine(x0, y0 - r, 2 * r + 1);
  fillCircleHelper(x0, y0, r, 3, 0);
}

void fillCircleHelper(int8_t x0, int8_t y0, int8_t r,
                      uint8_t cornername, int8_t delta) {

  int8_t f     = 1 - r;
  int8_t ddF_x = 1;
  int8_t ddF_y = -2 * r;
  int8_t x     = 0;
  int8_t y     = r;

  while (x < y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f     += ddF_y;
    }
    x++;
    ddF_x += 2;
    f     += ddF_x;

    if (cornername & 0x1) {
      drawFastVLine(x0 + x, y0 - y, 2 * y + 1 + delta);
      drawFastVLine(x0 + y, y0 - x, 2 * x + 1 + delta);
    }
    if (cornername & 0x2) {
      drawFastVLine(x0 - x, y0 - y, 2 * y + 1 + delta);
      drawFastVLine(x0 - y, y0 - x, 2 * x + 1 + delta);
    }
  }
}


void drawFastVLine(int8_t x, int8_t y,
                   int8_t h) {
  drawLine(x, y, x, y + h - 1);
}

void drawFastHLine(int8_t x, int8_t y,
                   int8_t w) {
  drawLine(x, y, x + w - 1, y);
}

void fillRect(int8_t x, int8_t y, int8_t w, int8_t h) {
  for (int8_t i = x; i < x + w; i++) {
    drawFastVLine(i, y, h);
  }
}

void fillScreen() {
  fillRect(0, 0, 8, 8);
}

void clearScreen() {
  saveColor();
  red = bg_red;
  green = bg_green;
  blue = bg_blue;
  fillRect(0, 0, 8, 8);
  restoreColor();
}


void drawChar(int8_t x, int8_t y, unsigned char c) {

  if (c >= 176) c++; // Handle 'classic' charset behavior

  for (int8_t i = 0; i < 6; i++ ) {
    uint8_t line;
    if (i == 5)
      line = 0x0;
    else
      line = pgm_read_byte(font + (c * 5) + i);

    for (int8_t j = 0; j < 8; j++) {
      if (line & 0x1)
        plot(x + i, 8 + y - j);
      else if ( bg_red != red || bg_green != green || bg_blue != blue)
        bg_plot(x + i, 8 + y - j);

      line >>= 1;
    }
  }
}

void drawCharTransparent(int8_t x, int8_t y, unsigned char c) {

  if (c >= 176) c++; // Handle 'classic' charset behavior

  for (int8_t i = 0; i < 6; i++ ) {
    uint8_t line;
    if (i == 5)
      line = 0x0;
    else
      line = pgm_read_byte(font + (c * 5) + i);

    for (int8_t j = 0; j < 8; j++) {
      if (line & 0x1)
        plot(x + i, 8 + y - j);

      line >>= 1;
    }
  }
}

void getChar(String buffer[], unsigned char c, unsigned char col, int8_t x, int8_t y) {


  for (int8_t yi = 0; yi < 8; yi++)
    for (int8_t xi = 0; xi < 8; xi++)
      buffer[xi][yi] = '-';

  if (c >= 176) c++; // Handle 'classic' charset behavior

  for (int8_t i = 0; i < 6; i++ ) {
    uint8_t line;
    if (i == 5)
      line = 0x0;
    else
      line = pgm_read_byte(font + (c * 5) + i);

    for (int8_t j = 0; j < 8; j++) {
      if (x + i >= 0 && x + i < 8 && y + j >= 0 && y + j < 8) {
        if (line & 0x1)
          buffer[y + j][x + i] = col;
        else
          buffer[y + j][x + i] = '-';
      }
      line >>= 1;
    }
  }
}


void scroll(String text, int scrollspeed, int offset) {

  int numberofcharacters = text.length();

  for (int xv = 7; xv > -((numberofcharacters) * 6) ; xv--) {

    clearScreen();

    for (int index = 0; index < numberofcharacters ; index++) {
      drawChar(xv + 6 * index, offset, text.charAt(index));
    }

    Colorduino.FlipPage();
    delay(scrollspeed);

  }
}


void scrollOverImg(String text, String img[], int scrollspeed, int offset) {

  int numberofcharacters = text.length();

  for (int xv = 7; xv > -((numberofcharacters) * 6) ; xv--) {

    clearScreen();
    drawImage(img, 0, 0);

    for (int index = 0; index < numberofcharacters ; index++) {
      drawCharTransparent(xv + 6 * index, offset, text.charAt(index));
    }

    Colorduino.FlipPage();
    delay(scrollspeed);

  }
}

void drawImage(String img[], int8_t x, int8_t y) {

  int index = 0;

  for (int8_t yi = 0; yi < 8; yi++) {
    for (int8_t xi = 0; xi < 8; xi++) {

      uint8_t c = img[yi][xi];
      if (c != ' ' && c != '\n' ) {
        if (c == '-')
          bg_plot(xi, 7 - yi);

        else {

          uint8_t ci = c - 48;

          saveColor();
          setColor(ci);
          plot(xi, 7 - yi);
          restoreColor();

        }
      }
    }
  }

}

void drawImageTransparent(String img[], int8_t x, int8_t y) {

  int index = 0;

  for (int8_t yi = 0; yi < 8; yi++) {
    for (int8_t xi = 0; xi < 8; xi++) {

      uint8_t c = img[yi][xi];
      if (c != ' ' && c != '\n' ) {
        if (c != '-') {

          uint8_t ci = c - 48;

          saveColor();
          setColor(ci);
          plot(xi, 7 - yi);
          restoreColor();

        }
      }
    }
  }

}

void rotateLeft(String img[], String tempImage[], String returnImage[]) {

  for (int8_t yi = 0; yi < 8; yi++) {
    for (int8_t xi = 0; xi < 8; xi++) {

      char c = img[yi][xi];
      tempImage[yi][xi] = c;
    }
  }

  for (int8_t yi = 0; yi < 8; yi++) {
    for (int8_t xi = 0; xi < 8; xi++) {

      char c = tempImage[yi][xi];
      returnImage[7 - xi][yi] = c;
    }
  }
}

void scrollDown(String img[], String tempImage[], String returnImage[], uint8_t bgi) {

  uint8_t bg = bgi + 48; // convert to '0', '1', ...

  for (int8_t yi = 0; yi < 8; yi++) {
    for (int8_t xi = 0; xi < 8; xi++) {

      char c = img[yi][xi];
      tempImage[yi][xi] = c;
    }
  }

  for (int8_t yi = 0; yi < 8; yi++) {
    for (int8_t xi = 0; xi < 8; xi++) {
      returnImage[yi][xi] = '-';
    }
  }

  for (int8_t yi = 1; yi < 8; yi++) {
    for (int8_t xi = 0; xi < 8; xi++) {

      char c = tempImage[yi - 1][xi];
      if ( c != bg && c != '-')
        returnImage[yi][xi] = c;
    }
  }

  for (int8_t i = 0; i < 8; i++)
    returnImage[0][i] = bg;

}

void scrollLeft(String img[], String tempImage[], String returnImage[], uint8_t bgi) {

  uint8_t bg = bgi + 48; // convert to '0', '1', ...

  for (int8_t yi = 0; yi < 8; yi++) {
    for (int8_t xi = 0; xi < 8; xi++) {

      char c = img[yi][xi];
      tempImage[yi][xi] = c;
    }
  }

  for (int8_t yi = 0; yi < 8; yi++) {
    for (int8_t xi = 0; xi < 8; xi++) {
      returnImage[yi][xi] = '-';
    }
  }

  for (int8_t yi = 0; yi < 8; yi++) {
    for (int8_t xi = 1; xi < 8; xi++) {

      char c = tempImage[yi][xi-1];
      if ( c != bg && c != '-')
        returnImage[yi][xi] = c;
    }
  }

  for (int8_t i = 0; i < 8; i++)
    returnImage[i][0] = bg;

}

void letItSnow(String img[], String tempImage[], String returnImage[]) {

  unsigned long time = millis();

  for (int8_t yi = 1; yi < 8; yi++) {
    for (int8_t xi = 0; xi < 8; xi++) {
      tempImage[yi][xi] = '-';
    }
  }
  
  for (int x = 0; x < 8; x++) {
    if (random(4) == 0)
      tempImage[0][x] = '9';
  }

  clearScreen();
  
  drawImage(img, 0, 0);
  drawImageTransparent(tempImage, 0, 0);
  Colorduino.FlipPage();
  
  //
  //
  //

  String* img0 = tempImage;

  while (millis() - time < 30000) {

    scrollDown(img0, tempImage, returnImage, 8);
    for (int x = 0; x < 8; x++) {
      if (random(4) == 0)
        img0[0][x] = '9';
      else
        img0[0][x] = '-';
    }

    if (millis() - time > 5000) {
      img0[7][1] = '9';
      img0[7][2] = '9';
      img0[7][3] = '9';
      img0[7][6] = '9';
    }
    if (millis() - time > 10000) {
      img0[7][0] = '9';
      img0[7][5] = '9';
    }
    if (millis() - time > 15000) {
      img0[7][4] = '9';
      img0[7][7] = '9';
      img0[7][6] = '9';
    }
    if (millis() - time > 20000) {
      img0[6][1] = '9';
      img0[6][2] = '9';
      img0[6][3] = '9';
      img0[6][6] = '9';
    }
    if (millis() - time > 20000) {
      img0[5][1] = '9';
      img0[5][2] = '9';
      img0[5][3] = '9';
      img0[5][6] = '9';
      img0[6][0] = '9';
      img0[6][5] = '9';
    }
    
    
    clearScreen();
    drawImage(img, 0, 0);

    
    drawImageTransparent(returnImage,  0, 0);
    Colorduino.FlipPage();
    delay(100);
    img0 = returnImage;
  }

}


void saveColor() {

  red_saved = red;
  green_saved = green;
  blue_saved = blue;

}

void restoreColor() {

  red = red_saved;
  green = green_saved;
  blue = blue_saved;

}


void setColor(uint8_t ci) {

  red   = colors[ci][0];
  green = colors[ci][1];
  blue  = colors[ci][2];

}


boolean isBgColor(uint8_t ci) {

  int red   = colors[ci][0];
  int green = colors[ci][1];
  int blue  = colors[ci][2];

  return  red == bg_red && green == bg_green && blue == bg_blue;

}


void setBgColor(uint8_t ci) {

  bg_red   = colors[ci][0];
  bg_green = colors[ci][1];
  bg_blue  = colors[ci][2];

}

void dropImage(String img1[], String tempImage[], String returnImage[]) {

  clearScreen();
  scrollDown(img1, tempImage, returnImage, 8);
  drawImage(returnImage,  0, 0);
  Colorduino.FlipPage();
  delay(100);

  for (int i = 0; i < 8; i++) {
    clearScreen();
    scrollDown(returnImage, tempImage, returnImage, 8);
    drawImage(returnImage,  0, 0);
    Colorduino.FlipPage();
    delay(100);
  }
}


void wipeImageLeft(String img1[], String tempImage[], String returnImage[]) {

  clearScreen();
  scrollLeft(img1, tempImage, returnImage, 8);
  drawImage(returnImage,  0, 0);
  Colorduino.FlipPage();
  delay(100);

  for (int i = 0; i < 8; i++) {
    clearScreen();
    scrollLeft(returnImage, tempImage, returnImage, 8);
    drawImage(returnImage,  0, 0);
    Colorduino.FlipPage();
    delay(100);
  }
}

void spinImageLeft(String img1[], String tempImage[], String returnImage[]) { 
    
  rotateLeft(img1, tempImage, returnImage);
  clearScreen();
  drawImage(returnImage,  0, 0);
  Colorduino.FlipPage();
  delay(100);

  for (int i = 0; i < 15; i++) {
    rotateLeft(returnImage, tempImage, returnImage);
    clearScreen();
    drawImage(returnImage,  0, 0);
    Colorduino.FlipPage();
    delay(100);
  }
}


void setup()
{
  Colorduino.Init();
  // compensate for relative intensity differences in R/G/B brightness
  // array of 6-bit base values for RGB (0~63)
  // whiteBalVal[0]=red
  // whiteBalVal[1]=green
  // whiteBalVal[2]=blue
  unsigned char whiteBalVal[3] = {36,63,63}; // for LEDSEE 6x6cm round matrix
  Colorduino.SetWhiteBal(whiteBalVal);
}

void loop()
{

  String returnImage[] = {
    "--------",
    "--------",
    "--------",
    "--------",
    "--------",
    "--------",
    "--------",
    "--------"
  };

  String tempImage[] = {
    "--------",
    "--------",
    "--------",
    "--------",
    "--------",
    "--------",
    "--------",
    "--------"
  };

  String img1[] = {
    "--------",
    "--------",
    "--------",
    "--------",
    "--------",
    "--------",
    "--------",
    "--------"
  };

  //
  //
  //
  
  setBgColor(8);
  setColor(3); 
  clearScreen();
  drawImage(heart,  0, 0);
  Colorduino.FlipPage();
  delay(3500);
  wipeImageLeft(heart, tempImage, returnImage); 

  String message = "Hello, nice to meet you!"; 

  setBgColor(8);
  setColor(3);
  clearScreen();
  Colorduino.FlipPage();
  scroll(message, 40, -1);

  setBgColor(5);
  setColor(3);
  clearScreen();
  Colorduino.FlipPage();
  scroll(message, 40, -1);  
  
  setBgColor(8);
  setColor(3);
  clearScreen();
  drawImage(heart,  0, 0);
  Colorduino.FlipPage();
  delay(1500);
  wipeImageLeft(heart, tempImage, returnImage); 

  setBgColor(8);
  setColor(1);
  clearScreen();
  Colorduino.FlipPage();
  scroll(message, 40, -1);  
  
  setBgColor(8);
  clearScreen();  
  Colorduino.FlipPage();
  getChar(img1, '!', '3', 2, 0);
  setColor(3); 
  drawImage(img1,  0, 0);
  Colorduino.FlipPage();
  delay(1500);
        
  spinImageLeft(img1, tempImage, returnImage);   
  wipeImageLeft(img1, tempImage, returnImage); 

  setBgColor(8);
  clearScreen();  
  Colorduino.FlipPage();
  getChar(img1, '*', '5', 0, 0);
  setColor(3); 
  scrollOverImg(message, img1, 40, -1);

  setBgColor(8);
  clearScreen();  
  Colorduino.FlipPage();
  getChar(img1, '*', '5', 1, 0);
  setColor(3); 
  scrollOverImg(message, img1, 40, -1);
  
  clearScreen();
  drawImage(heart,  0, 0);
  Colorduino.FlipPage();
  delay(1500);
      
  letItSnow(heart, tempImage, returnImage);
  delay(1500);
  dropImage(heart, tempImage, returnImage); 

  clearScreen();
  drawImage(heart,  0, 0);
  Colorduino.FlipPage();
  delay(3500);
  
  wipeImageLeft(heart, tempImage, returnImage); 

  clearScreen();
  drawImage(hi,  0, 0);
  Colorduino.FlipPage();
  delay(3500);

  spinImageLeft(hi, tempImage, returnImage); 
  clearScreen();

  setBgColor(8);
  setColor(5);
  clearScreen();  
  Colorduino.FlipPage();
  scrollOverImg(message, heart, 40, -1);
  setColor(3);
  scrollOverImg(message, heart, 40, -1);

  clearScreen();
  drawImage(heart,  0, 0);
  Colorduino.FlipPage();
  delay(3500);
  
  wipeImageLeft(heart, tempImage, returnImage); 

  for (int i = 0; i <= 100; i ++)  {
    clearScreen(); 
    setColor(random(9)); 
    fillCircle(random(8), random(8), random(5)+1); 
    Colorduino.FlipPage();
    delay(100);     
  }


  for (int i = 0; i <= 100; i ++)  {
    if (i % 5 == 0)
      clearScreen();     
    setColor(random(9)); 
    drawLine(random(8), random(8), random(8), random(8)); 
    Colorduino.FlipPage();
    delay(100);     
  }

  
  for (int i = 0; i <= 1000; i ++)  {
    if (i % 10 == 0)
      clearScreen(); 
    setColor(random(9)); 
    plot(random(8), random(8)); 
    Colorduino.FlipPage();
    delay(10);     
  }
  
  
  setBgColor(5);
  clearScreen();
  drawImage(hi,  0, 0);
  Colorduino.FlipPage();
  delay(3500);
  
  wipeImageLeft(heart, tempImage, returnImage); 
  delay(1500);

}
